# -*- coding: utf-8 -*-

# Handles writing and loading XML configuration files

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import codecs
import time
import re
from os import path
from xml.dom import minidom
from xml.sax.saxutils import escape
from xml.sax.saxutils import unescape


import utils
import operations


# Get values, write configuration file
def saveConfig(main):
    # save dialog
    dlg = wx.FileDialog(None, message=_(u"Save configuration as ..."),
          defaultDir=utils.realPath(u'configs'), defaultFile=u'.cfg',
          wildcard=_(u"Configuration file (*.cfg)")+u'|*.cfg',
          style=wx.SAVE|wx.OVERWRITE_PROMPT
          )
    # write file
    if dlg.ShowModal() == wx.ID_OK:
        cfgFilePath = dlg.GetPath()
        cfgFileExt = path.splitext(cfgFilePath)[1]
        if cfgFileExt != '.cfg':
            cfgFilePath = cfgFilePath + '.cfg'
        # create and write the file
        utils.writeFile(cfgFilePath, SaveConfig(main).cfgFile)
    dlg.Destroy()


# Get config file path from dialog and apply settings
def loadConfig(main, configFilePath=False):
    if configFilePath:
        LoadConfig(main, configFilePath)
    else:
        dlg = wx.FileDialog(None,
          message=_(u"Load a configuration file"),
          defaultDir=utils.realPath(u'configs'), defaultFile=u'',
          wildcard=_(u"Configuration file (*.cfg)")+u'|*.cfg',
          style=wx.OPEN
          )
        if dlg.ShowModal() == wx.ID_OK:
            LoadConfig(main, dlg.GetPath())
        dlg.Destroy()


class SaveConfig():

    def __init__(self, mainWindow):
        global main
        main = mainWindow
        self.cfgFile = self.createConfigFile()

    # Get correct values depending on child type.
    def getChildValues(self, child):
        value = None
        if isinstance(child, wx.CheckBox) or isinstance(child, wx.RadioButton):
            value = int(child.GetValue())
        elif isinstance(child, wx.TextCtrl) or isinstance(child, wx.SpinCtrl):
            value = child.GetValue()
        elif isinstance(child, wx.ComboBox):
            value = child.GetValue()
        elif isinstance(child, wx.Choice):
            value = child.GetSelection()

        if value is not None:
            if type(value) == type(u''):
                value = escape(value)
            widgetType = child.GetClassName()
            id = child.GetName()
            return id, widgetType, value

    # Recursively get the complete path down hierachy to value.
    # End result is a XML structure of values.
    def allChildParamsToXml(self, cfg, child, r):
        # panels & notebook get children
        if isinstance(child, wx.Notebook) or isinstance(child, wx.Panel):
            type = child.GetClassName()
            id = child.GetName()
            cfg += '\t\t\t'+r*'\t'+u'<%s id="%s">\n'%(type,id)
            for child in child.GetChildren():
                try:
                    cfg = self.allChildParamsToXml(cfg, child, r+1)
                except TypeError:
                    pass
            cfg += '\t\t\t'+r*'\t'+u"</%s>\n"%type
            return cfg
        # others get values
        else:
            try:
                id, widgetType, value = self.getChildValues(child)
            except TypeError:
                pass
            else:
                cfg += '\t\t\t'+r*'\t'+u'<value id="%s" type="%s">%s</value>\n'\
                      %(id, widgetType, value)
            return cfg

    # Convert all renaming operations to XML, including parameters and widget values
    def renamingOperationsToXml(self):
        cfg = ''
        operationList = main.renamer.Panel.usedOperations
        operationStack = main.renamer.operations

        for i in range(operationList.GetItemCount()):
            type = operationList.GetItemText(i)
            type = re.sub("\d{1,}: ", '', type)
            type = operations.defs[type][0]

            operation = operationStack[i]

            cfg += u'\t\t<operation id="%s" type="%s">\n'%(i, type)
            cfg += u'\t\t\t<parameters>\n'

            for k in operation.params.keys():
                v = operation.params[k]
                cfg += u'\t\t\t\t<%s>%s</%s>\n' % (k, v, k)

            cfg += '\t\t\t</parameters>\n'

            for child in operation.GetChildren():
                cfg = self.allChildParamsToXml(cfg, child, 0)

            cfg += u'\t\t</operation>\n'

        return cfg

    # Create the XML config file
    def createConfigFile(self):
        datetime = utils.uDate(main, '%Y-%m-%d %H:%M:%S', time.localtime())
        cfgFile = u'<?xml version="1.0" encoding="UTF-8"?>\n'
        cfgFile += u'<configuration application="Métamorphose-2" '+\
                   'version="%s" datetime="%s">\n'\
                   %(main.version, datetime)
        # get info for 'normal' notebook tabs
        pages = utils.getNotebookPages(main)
        for i in (0,2,3):
            # section header is page name
            cfgFile += u'\t<page id="%s" name="%s">\n'%(i,pages[i].GetName())

            # TODO needs work - doesn't do tree, use allChildParamsToXml?
            for child in pages[i].GetChildren():
                try:
                    id,type,value = self.getChildValues(child)
                except TypeError:
                    pass
                else:
                    cfgFile += u'\t\t<value id="%s" type="%s">%s</value>\n'\
                               %(id,type,value)
            cfgFile += u'\t</page>\n'

        # get renaming operations
        cfgFile += u'\t<page id="1" name="%s">\n'%pages[1].GetName()
        cfgFile += self.renamingOperationsToXml()
        cfgFile += "\t</page>\n</configuration>\n"
        
        return cfgFile




class LoadConfig():

    def __init__(self, mainWindow, configFilePath):
        global main
        main = mainWindow
        self.loadConfig(configFilePath)

    # Set widgets values correctly depending on type
    def setWidgetValues(self, id, type, value):
        if value == None:
            value = ''
        else:
            value = unescape(value)

        if type == 'wxCheckBox' or type == 'wxRadioButton' or type == 'wxSpinCtrl':
            id.SetValue(int(value))
        elif type == 'wxTextCtrl' or type == 'wxComboBox':
            id.SetValue(value)
        elif type == 'wxChoice':
            id.SetSelection(int(value))


    # Get widget values recursively through XML tree,
    # apply setings to operation
    def applyValues(self, parent, node):
        id = node.attributes['id'].value
        # value, apply
        if node.tagName == 'value':
            wtype = node.attributes['type'].value
            try:
                value = node.childNodes[0].nodeValue
            except IndexError:
                value = None
            try:
                widget = getattr(parent, id)
            except AttributeError:
                pass
            else:
                self.setWidgetValues(widget, wtype, value)
        # notebook, cycle through pages
        elif node.tagName == 'wxNotebook':
            notebook = getattr(parent, id)
            names = []
            for i in range(notebook.GetPageCount()):
                names.append(notebook.GetPage(i).GetName())
            for child in node.getElementsByTagName('wxPanel'):
                childname = child.attributes['id'].value
                if childname in names:
                    self.applyValues(parent, child)
        # panel, cycle through panels and values
        elif node.tagName == 'wxPanel':
            #print parent, id
            parent = getattr(parent, id)
            for child in node.childNodes:
                if child.nodeType == 1:
                    self.applyValues(parent, child)


    # Get operation parameters from XML, return dict
    def getOperationParams(self, op):
        params = {}
        try:
            paramNode = op.getElementsByTagName('parameters')[0]
        except IndexError:
            utils.debugPrint(main,
                u"warning : could not load all parameters for '%s' operation." % op.attributes['type'].value
            )
        else:
            for node in paramNode.childNodes:
                if node.nodeType == 1:
                     value = node.childNodes[0].nodeValue
                     if value == 'True': value = True
                     if value == 'False': value = False
                     params[node.nodeName] = value
        return params


    def loadConfigXML(self, config):
        pages = utils.getNotebookPages(main)
        configVersion = config.attributes['version'].value

        utils.debugPrint(main, "config version : %s"%configVersion)

        for page in config.getElementsByTagName('page'):
            page_name = page.attributes['name'].value
            page_id = int(page.attributes['id'].value)

            # normal pages
            if page_name != 'mainPanel':
                utils.debugPrint(main, "loading config page : %s"%page_name)

                for node in page.getElementsByTagName('value'):
                    id = node.attributes['id'].value
                    # make sure element exists before setting it
                    if hasattr(pages[page_id], id):
                        id = getattr(pages[page_id], id)
                    else:
                        continue
                    type = node.attributes['type'].value
                    try:
                        value = node.childNodes[0].nodeValue
                    except IndexError:
                        value = None
                    self.setWidgetValues(id, type, value)
                    # adjustments if needed
                    if hasattr(pages[page_id], 'onConfigLoad'):
                        pages[page_id].onConfigLoad()
            # main page
            else:
                utils.debugPrint(main, "loading renamer config")
                # cleanup before loading
                main.renamer.Panel.destroyAllOperations()

                # load values
                for op in page.getElementsByTagName('operation'):
                    type = op.attributes['type'].value
                    type = operations.getTranslatedNameFromType(type)
                    utils.debugPrint(main, "loading operation type : %s"%type)

                    # make sure it's a valid type
                    if type == False:
                        continue
                    id = op.attributes['id'].value

                    # initialise object, passing parameters
                    params = self.getOperationParams(op)
                    main.renamer.Panel.addOperationToStack(type, id, params)

                    # apply node values to widgets
                    operation = main.renamer.operations[int(id)]
                    for node in op.childNodes:
                        if node.nodeType == 1 and node.nodeName != "parameters":
                            self.applyValues(operation, node)

                    # apply each operation's post load routine
                    if hasattr(operation, 'onConfigLoad'):
                        operation.onConfigLoad()


    # Read file and apply settings
    def loadConfig(self, configFilePath):
        utils.debugPrint(main, "loading config file : %s"%configFilePath)
        # attempt to open config file
        try:
            xmldoc = codecs.open(configFilePath,'r', 'utf-8')
            xmldoc = minidom.parseString(xmldoc.read().encode("utf-8"))
        except:
            utils.makeErrMsg(
                _(u"Invalid XML in config file : %s") % configFilePath,
                _(u"Invalid Configuration")
            )
        else:
            config = xmldoc.firstChild

            # set autopreview to false while loading config
            # avoids loading paths more than once
            v = main.bottomWindow.autoPreview.GetValue()
            main.bottomWindow.autoPreview.SetValue(False)

            # get original path
            oldPath = main.picker.Panel.path.GetValue()

            self.loadConfigXML(config)

            # Do not replace a set path if one is not set in the config
            newPath = main.picker.Panel.path.GetValue()            
            if oldPath and not newPath:
                main.picker.Panel.path.SetValue(oldPath)

            # preview
            main.bottomWindow.autoPreview.SetValue(v)
            if main.autoModeLevel != 0 or\
             (main.prefs.get('previewOnConfig') and main.autoModeLevel is False):
                main.picker.Panel.resetDirPickerOnConfigLoad()
                
